{{ NordicSpiAsmDemoLean110718c

  Modified from NordicSpiAsmDemo110718c
  110718c strip the program down to
  bare neccessities.
  Demo program to receive data
  from Nordic FOBs using SPI_Asm.
  By: Duane Degn
 
}}
CON

  'nRF24L01 constants:
  
  'Commands
  R_REGISTER = %0000_0000 '+ reg
  W_REGISTER = %0010_0000 '+ reg
  R_RX_PAYLOAD = %0110_0001
  W_TX_PAYLOAD = %1010_0000
  FLUSH_TX = %1110_0001
  FLUSH_RX = %1110_0010
  REUSE_TX_PL = %1110_0011
  R_RX_PL_WID = %0110_0000
  W_ACK_PAYLOAD = %1010_1000 '+ pipe
  W_TX_PAYLOAD_NOACK = %1011_0000
  NOOP = %1111_1111

  'Registers
  CONFIG = $00   '%0000_1000 / $08
  EN_AA = $01  '%0011_1111  / $3F
  EN_RXADDR = $02  '%0000_0011
  SETUP_AW = $03   '%0000_0011
  SETUP_RETR = $04  '%0000_0011
  RF_CH = $05   '%0000_0010
  RF_SETUP = $06  '%0000_1110
  _STATUS = $07     '%0000_1110 / $E 
  OBSERVE_TX = $08  '%0000_0000
  RPD = $09   '%0000_0000 
  RX_ADDR_P0 = $0A '$E7E7E7E7E7
  RX_ADDR_P1 = $0B '$C2C2C2C2C2
  RX_ADDR_P2 = $0C  '$C3
  RX_ADDR_P3 = $0D  '$C4 
  RX_ADDR_P4 = $0E  '$C5 
  RX_ADDR_P5 = $0F  '$C6 
  TX_ADDR = $10    '$E7E7E7E7E7
  RX_PW_P0 = $11   '%0000_0000
  RX_PW_P1 = $12   '%0000_0000 
  RX_PW_P2 = $13   '%0000_0000 
  RX_PW_P3 = $14  '%0000_0000 
  RX_PW_P4 = $15  '%0000_0000 
  RX_PW_P5 = $16  '%0000_0000 
  FIFO_STATUS = $17 '%0001_0001 
  DYNPD = $1C   '%0000_0000
  FEATURE = $1D  '%0000_0000

CON

  _CLKMODE = XTAL1 + PLL16X
  _CLKFREQ = 80_000_000


  'Pins to Nordic nRF24L01+
  _SpiIrq = 12
  _SpiMiso = 10
  _SpiMosi = 8
  _SpiSck = 6 
  _SpiCsn = 4     
  _SpiCe = 2
  
  _RxLed = 23   ' far left LED on QuickStart board
  
  _DebugBaud = 57600

OBJ 

  Spi : "SPI_Asm"
  Debug : "fullduplexserial"

VAR

  long configPtr

  byte globalPayload[5]
  byte payloadSize, globalAddressSize
  byte statusReset
  byte rxPayload[4]
  byte rxAddress[5], txAddress[5]
  
PUB Main | loopCount, wrongStatusCount, localIndex, zeroPayloadCount

  Debug.start(31, 30, 0, _DebugBaud)
  waitcnt(clkfreq / 1000 + cnt) ' wait for serial driver cog to start
  
  outa[_SpiSck]~
  outa[_SpiMosi]~
  outa[_SpiCsn]~~                          ' chip select not, active low
  outa[_SpiCe]~
  
  dira[_SpiSck]~~
  dira[_SpiMosi]~~
  dira[_SpiCsn]~~
  dira[_SpiCe]~~

  Spi.start(5, 0)
  waitcnt(clkfreq / 10 + cnt)

  Configure(@configShort)           
  
  repeat
    ListenForFob
    waitcnt(clkfreq / 1000 + cnt) 'I don't know if this delay is needed.
    
PUB ListenForFob | localIndex

  if ina[_SpiIrq]  == 0
    outa[_RxLed]~~ 
    ReadPayload(@rxPayload)   
    Debug.str(String(13, "rxPayload = $"))
    repeat localIndex from 0 to 3
      Debug.hex(rxPayload[localIndex], 2)
      if localIndex <> 3
        Debug.str(String(", $"))
    waitcnt(clkfreq / 500 + cnt) 'I don't know if this delay is needed.
    outa[_RxLed]~
     
PUB Status

  outa[_SpiMosi]~~
  outa[_SpiCsn]~  
  result := SpiRead
  outa[_SpiCsn]~~
  outa[_SpiMosi]~
  
  'Reset IRQ 
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + _STATUS)
  SpiWrite(statusReset)  
  outa[_SpiCsn]~~
   
PUB Configure(localPtr) | localIndex
{{
  Configure nRF24L01 registers for receive mode.
  Using data at location "localPtr" provided
  by parent object.
  The data should be arranged so the first seven bytes starting
  at localPtr  contain data to be written to the first seven
  registers of this device.  The eight byte should be payload
  size.  The nineth byte should be the start of the RX address
  followed by the TX address.
  To change the configuration, either the data in the parent object
  could be overwritten with the new desired data or a pointer
  to a different set of data could be used as the parameter
  when calling this object.  
 
}}
  ' Load global variables with requently used data.
  
  configPtr := localPtr
  payloadSize := byte[configPtr + _PayloadOffset] '_payloadSize
  statusReset := byte[configPtr + _StatusOffset]

  globalAddressSize := byte[configPtr + _AddressSizeOffset]
  globalAddressSize += 2
  globalAddressSize <#= 5
  globalAddressSize #>= 3            
              
  repeat localIndex from 0 to globalAddressSize - 1
    rxAddress[localIndex] := byte[configPtr + _RxAddressOffset + localIndex]
    txAddress[localIndex] := byte[configPtr + _RxAddressOffset + globalAddressSize + localIndex]

  ' Write the registers with the new data.
  outa[_SpiCe]~
   
   
  outa[_SpiCsn]~   
  SpiWrite(W_REGISTER + EN_AA)
  SpiWrite(byte[localPtr + EN_AA])
  outa[_SpiCsn]~~
   
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + SETUP_RETR)
  SpiWrite(byte[localPtr + SETUP_RETR])    
  outa[_SpiCsn]~~    
  
  outa[_SpiCsn]~  
  SpiWrite(W_REGISTER + SETUP_AW)   
  SpiWrite(byte[localPtr + SETUP_AW])   
  outa[_SpiCsn]~~    
  
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + RF_SETUP) 
  SpiWrite(byte[localPtr + RF_SETUP])  
  outa[_SpiCsn]~~

  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + RX_PW_P0) 
  SpiWrite(payloadSize)
  outa[_SpiCsn]~~   
  
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + RF_CH)
  SpiWrite(byte[localPtr + RF_CH]) ' *** this is different than earlier driver  
  outa[_SpiCsn]~~                    ' I think it fixes a bug that kept it from communicating
                                    ' with the older modules.
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + RX_ADDR_P0)
  repeat localIndex from 0 to globalAddressSize - 1
    SpiWrite(rxAddress[localIndex]) 
  outa[_SpiCsn]~~
  
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + TX_ADDR)
  repeat localIndex from 0 to globalAddressSize - 1
    SpiWrite(txAddress[localIndex]) 
  outa[_SpiCsn]~~  

  outa[_SpiCsn]~  
  SpiWrite(W_REGISTER + CONFIG)
  SpiWrite(byte[localPtr])
  outa[_SpiCsn]~~
    
  'Start receiving
  outa[_SpiCe]~~

PUB ReadPayload(stringToFillPtr) | localIndex
{{
  Reads payload from nRF24L01.
  Also flushes RX FIFO and resets IRQ state
}}    
 
  'Stop receiving
  outa[_SpiCe]~
    
  'Read RX payload   
  outa[_SpiCsn]~    
  SpiWrite(R_RX_PAYLOAD)
  outa[_SpiMosi]~~
  repeat localIndex from 0 to payloadSize - 1
    byte[stringToFillPtr++] := SpiRead
  outa[_SpiCsn]~~
  outa[_SpiMosi]~
    
  'Flush RX FIFO    
  outa[_SpiCsn]~
  SpiWrite(FLUSH_RX)    
  outa[_SpiCsn]~~

  'Reset IRQ 
  outa[_SpiCsn]~
  SpiWrite(W_REGISTER + _STATUS)
  SpiWrite(statusReset)   
  outa[_SpiCsn]~~

  'Start receiving
  outa[_SpiCe]~~
  
PUB SpiRead
                        
  result := Spi.ShiftIn(_SpiMiso, _SpiSck, Spi#MSBPRE, 8)
   
PUB SpiWrite(byteOut)

  Spi.ShiftOut(_SpiMosi, _SpiSck, Spi#MSBFIRST, 8, byteOut)   
     
PUB SpiReadMulti(localCommand, localPtr, bytesToRead) | localIndex

  bytesToRead <#= 5
  outa[_SpiCsn]~
  SpiWrite(localCommand)
  outa[_SpiMosi]~~
  repeat localIndex from 0 to bytesToRead - 1
    byte[localPtr + localIndex] := SpiRead
  outa[_SpiCsn]~~
  outa[_SpiMosi]~
   
PUB SpiWriteMulti(localRegister, localPtr, bytesToWrite) | localIndex 

  bytesToWrite <#= 5
  outa[_SpiCsn]~ 
  SpiWrite(localRegister + W_REGISTER)
  repeat localIndex from 0 to bytesToWrite - 1
    SpiWrite(byte[localPtr + localIndex])
  outa[_SpiCsn]~~
        
CON'' Nordic FOB Constants

  ' Same order as "nordic-nRF24L01.c" source code
  _FobReg0x20 = $78 ' %01111000
  _FobReg0x21 = $00 ' %00000000
  _FobReg0x24 = $00 ' %00000000 
  _FobReg0x23 = $03 ' %00000011 ' five digit address
  _FobReg0x26 = $07 ' %00000111 ' 1Mbit, 0dBm power, zero bit don't care same as $06
  _FobReg0x25 = $02 ' %00000010
  

CON'' Prop Listen for Nordic FOB Constants

  _ListenForFobReg0 = _FobReg0x20 - %01000000 + %00000011' add irq, power up and RRX
  _ListenForFobReg1 = _FobReg0x21
  _ListenForFobReg2 = %00000011
  _ListenForFobReg3 = _FobReg0x23
  _ListenForFobReg4 = _FobReg0x24
  _ListenForFobReg5 = _FobReg0x25
  _ListenForFobReg6 = _FobReg0x26
  _ListenForFobRxPayload = 4

CON

  _InitialAddressCharacter = $E7
  _NordicStatus = %01000000

  ' Memory Locations of Config Data

  _AddressSizeOffset = 3
  _StatusOffset = 7
  _PayloadOffset = 8
  _RxAddressOffset = 9

DAT
''------------------ Config settings for new nRF24L01+ modules ----------------------------------------
''------------------ Config settings for receiving short FOB messages ----------------------------------------
'' ----------------- Keep below together and in order ------------------------------------------------
configShort   byte _ListenForFobReg0, _ListenForFobReg1, _ListenForFobReg2
              byte _ListenForFobReg3, _ListenForFobReg4, _ListenForFobReg5
              byte _ListenForFobReg6
nordicStatus  byte _NordicStatus
rxPayloadShort byte _ListenForFobRxPayload ' Register $11 RX_PW_P0

rxAddressShort byte _InitialAddressCharacter[5] ' Register $10 Transmit address data pipe 0, LSByte first
txAddressShort byte _InitialAddressCharacter[5] ' Register $10 Transmit address data pipe 0, LSByte first
'' ----------------- Keep above together and in order ------------------------------------------------
